﻿//
// Myriadbits Disk space analyzer 
// Find free space on your NTFS disk
// Copyright (C) 2014 Myriadbits, Jochem Bakker
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// For more information, contact me at: info@myriadbits.com
//
// This code was copied from :
// http://stackoverflow.com/questions/616718/how-do-i-get-common-file-type-icons-in-c
// Extended with DestroyIcon and some null checking
//


using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;

namespace Myriadbits.DiskSpaceAnalyzer
{
	public static class FileIcon
	{
		[DllImport("shell32.dll")]
		private static extern IntPtr SHGetFileInfo(string pszPath, uint dwFileAttributes, ref SHFILEINFO psfi, uint cbSizeFileInfo, uint uFlags);
		[StructLayout(LayoutKind.Sequential)]
		private struct SHFILEINFO
		{
			public IntPtr hIcon;
			public int iIcon;
			public uint dwAttributes;
			[MarshalAs(UnmanagedType.ByValTStr, SizeConst = 260)]
			public string szDisplayName;
			[MarshalAs(UnmanagedType.ByValTStr, SizeConst = 80)]
			public string szTypeName;
		};

		[System.Runtime.InteropServices.DllImport("user32.dll", CharSet = CharSet.Auto)]
		extern static bool DestroyIcon(IntPtr handle);

		private const uint SHGFI_LARGEICON			= 0x00000000; // 'Large icon
		private const uint SHGFI_SMALLICON			= 0x00000001; // 'Small icon
		private const uint SHGFI_OPENICON			= 0x00000002; // Open icon
		private const uint SHGFI_SHELLICONSIZE		= 0x00000004;
		private const uint SHGFI_PIDL				= 0x00000008;
		private const uint SHGFI_USEFILEATTRIBUTES	= 0x00000010;
		private const uint SHGFI_ADDOVERLAYS		= 0x00000020;
		private const uint SHGFI_OVERLAYINDEX		= 0x00000040;
		private const uint SHGFI_ICON				= 0x00000100;
		private const uint SHGFI_DISPLAYNAME		= 0x00000200;
		private const uint SHGFI_TYPENAME			= 0x00000400;
		private const uint SHGFI_ATTRIBUTES			= 0x00000800;
		private const uint SHGFI_ICONLOCATION		= 0x00001000;
		private const uint SHGFI_EXETYPE			= 0x00002000;
		private const uint SHGFI_SYSICONINDEX		= 0x00004000;
		private const uint SHGFI_LINKOVERLAY		= 0x00008000;
		private const uint SHGFI_SELECTED			= 0x00010000;
		private const uint SHGFI_ATTR_SPECIFIED		= 0x00020000;

		public static Icon GetLargeIcon(string file)
		{
			FileIcon.SHFILEINFO shinfo = new FileIcon.SHFILEINFO();
			IntPtr hImgLarge = FileIcon.SHGetFileInfo(file, 0, ref shinfo, (uint)Marshal.SizeOf(shinfo), FileIcon.SHGFI_ICON | FileIcon.SHGFI_LARGEICON);
			if (shinfo.hIcon != null)
			{
				Icon newIcon = Icon.FromHandle(shinfo.hIcon);
				//DestroyIcon(shinfo.hIcon); // Destroy the original icon handle (now managed by the Icon class)
				return newIcon;
			}
			return null;
		}


		/// <summary>
		/// Return the icon type
		/// </summary>
		/// <param name="file"></param>
		/// <returns></returns>
		public static string GetIconType(string file)
		{
			FileIcon.SHFILEINFO shinfo = new FileIcon.SHFILEINFO();
			IntPtr hImgLarge = FileIcon.SHGetFileInfo(file, 0, ref shinfo, (uint)Marshal.SizeOf(shinfo), FileIcon.SHGFI_TYPENAME);
			return shinfo.szTypeName;
		}


		/// <summary>
		/// Get a small icon of the file
		/// </summary>
		/// <param name="file"></param>
		/// <returns></returns>
		public static Icon GetSmallIcon(string file, bool openIcon = false)
		{
			FileIcon.SHFILEINFO shinfo = new FileIcon.SHFILEINFO();
			uint options = FileIcon.SHGFI_ICON | FileIcon.SHGFI_SMALLICON | FileIcon.SHGFI_ADDOVERLAYS;
			if (openIcon)
				options |= FileIcon.SHGFI_OPENICON;
			IntPtr hImg = FileIcon.SHGetFileInfo(file, 0, ref shinfo, (uint)Marshal.SizeOf(shinfo), options);
			if (shinfo.hIcon != null)
			{
				Icon newIcon = Icon.FromHandle(shinfo.hIcon);
				//DestroyIcon(shinfo.hIcon);// Destroy the original icon handle (now managed by the Icon class)
				return newIcon;
			}
			return null;
		}
	}
}
