﻿//
// NTFS - Myriadbits .NET NTFS library. 
// Fast MFT reader.
// Copyright (C) 2014 Myriadbits, Jochem Bakker
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// For more information, contact me at: info@myriadbits.com
//

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;


namespace Myriadbits.NTFS
{
	public class NTFSIndexRoot_IndexRecord : DataBufferReader
	{
		public NTFSIndexRoot_IndexRecord(DataBuffer buffer, int offset) 
			: base(buffer, 0, offset)
		{		
		}

		public ulong FileReferenceNumber { get { return this.GetULong(0x00) & NTFS.referenceMask; } }
		public ulong FileReferenceNumberOrg { get { return this.GetULong(0x00); } }
		public ushort SizeOfIndexEntry { get { return this.GetUShort(0x08); } }
		public ushort FilenameOffset { get { return this.GetUShort(0x0A); } }
		public ushort IndexFlags { get { return this.GetUShort(0x0C); } }

		public ulong CreateUniqueReferenceNumber(byte index0 = 0, byte index1 = 0) { return (this.GetULong(0x00) & NTFS.referenceMask) + ((ulong)index0 << 56) + ((ulong)index1 << 48); }

		public ulong ParentReferenceNumber { get { return this.GetULong(0x10) & NTFS.referenceMask; } }
		public ulong ParentReferenceNumberOrg { get { return this.GetULong(0x10); } }

		public DateTime FileCreationTime { get { return this.GetDateTime(0x18); } }
		public DateTime FileAlteredTime { get { return this.GetDateTime(0x20); } }
		public DateTime MFTChangedTime { get { return this.GetDateTime(0x28); } }
		public DateTime FileReadTime { get { return this.GetDateTime(0x30); } }

		public ulong AllocatedFileSize { get { return this.GetULong(0x38); } }
		public ulong RealFileSize { get { return this.GetULong(0x40); } }

		public ulong FileFlags { get { return this.GetULong(0x48); } }

		public byte FilenameLength { get { return this.GetByte(0x50); } }
		public byte FileNamespace { get { return this.GetByte(0x51); } }

		public string Filename { get { return this.GetString(0x52, this.FilenameLength * 2); } }

		public ulong DataSize
		{
			get
			{
				return this.RealFileSize & 0x0000FFFFFFFFFFFF;
			}
		}

		public override string ToString()
		{
			return string.Format("$INDEX_ENTRY: {0}", this.Filename);
		}
	}
}
